//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

@Entity
@Table(name = "annotation_group")
public class AnnotationGroup implements Serializable {
	protected int annotationGroupId;
	protected AnnotationSession session = null;//ManyToOne
	protected Set<AnnotationElement> annotations = new HashSet<AnnotationElement>(
			0);//OneToMany
	protected AnnotationElement rootElement = null;//OneToOne

	/** default constructor */
	public AnnotationGroup() {
	}

	/** minimal constructor */
	public AnnotationGroup(int annotationGroupId) {
		this.annotationGroupId = annotationGroupId;
	}

	public AnnotationGroup(AnnotationGroup source) {
		this();
		merge(source);
	}

	protected void merge(AnnotationGroup source) {
		this.annotationGroupId = source.annotationGroupId;
		this.session = source.session;
		this.annotations = source.annotations;
		this.rootElement = source.rootElement;
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationGroup(";
		if (annotationGroupId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationGroupId));
		}
		if (rootElement != null) {
			name = name.concat("); root: " + rootElement.toString() + ")");
		} else {
			name = name.concat(")");
		}
		return name;
	}

}
