//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

@Entity
@Table(name = "annotation_group")
public class AnnotationGroupDb extends AnnotationGroup {

	/** default constructor */
	public AnnotationGroupDb() {
		super();
	}

	/** minimal constructor */
	public AnnotationGroupDb(int annotationGroupId) {
		super(annotationGroupId);
	}

	public AnnotationGroupDb(AnnotationGroup source) {
		super(source);
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationGroupDb(";
		if (annotationGroupId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationGroupId));
		}
		if (rootElement != null) {
			name = name.concat("); root: " + rootElement.toString() + ")");
		} else {
			name = name.concat(")");
		}
		return name;
	}

	public void merge(AnnotationGroup changedProperties) {
		if (this.annotationGroupId != 0
				&& changedProperties.annotationGroupId != this.annotationGroupId) {
			throw new IllegalArgumentException(
					"Groups do not have the same Id!");
		}
		super.merge(changedProperties);
	}

	@Id
	@Column(name = "annotation_group_id", unique = true, nullable = false)
	@GeneratedValue(generator = "id_gen")
	public int getAnnotationGroupId() {
		return annotationGroupId;
	}

	public void setAnnotationGroupId(int annotationGroupId) {
		this.annotationGroupId = annotationGroupId;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	// TODO hier Cascade?
	@JoinColumn(name = "session_id")
	public AnnotationSessionDb getSession() {
		if (session == null) {
			return null;
		} else {
			if (session.getClass() != AnnotationSessionDb.class) {
				session = new AnnotationSessionDb(session);
			}
			return (AnnotationSessionDb) session;
		}
	}

	public void setSession(AnnotationSessionDb session) {
		this.session = session;
	}

	/**
	 * Warning: Changes on the returned Set will not affect this Set. Use
	 * setAnnotations after changing!
	 * 
	 * @return a Copy of the contained Set<AnnotationElement> as
	 *         Set<AnnotationElementDb>
	 */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "parent", cascade = { CascadeType.REMOVE })
	// not null?
	public Set<AnnotationElementDb> getAnnotations() {
		Set<AnnotationElementDb> annotations = new HashSet<AnnotationElementDb>();
		Iterator<AnnotationElement> it = this.annotations.iterator();
		while (it.hasNext()) {
			AnnotationElement next = it.next();
			if (next.getClass() != AnnotationElementDb.class) {
				next = new AnnotationElementDb(next);
			}
			annotations.add((AnnotationElementDb) next);
		}
		return annotations;
	}

	public void setAnnotations(Set<AnnotationElementDb> annotations) {
		this.annotations.clear();
		this.annotations.addAll(annotations);
	}

	@OneToOne(fetch = FetchType.EAGER, optional = true, cascade = { CascadeType.REMOVE })
	public AnnotationElementDb getRootElement() {
		if (rootElement == null) {
			return null;
		} else {
			if (rootElement.getClass() != AnnotationElementDb.class) {
				rootElement = new AnnotationElementDb(rootElement);
			}
			return (AnnotationElementDb) rootElement;
		}
	}

	public void setRootElement(AnnotationElementDb rootElement) {
		this.rootElement = rootElement;
	}

}
